//******************************************************************************
//******************************************************************************
/**
* @file    IBPNGClient.h
* @author  Markus van Pinxteren
* @date    21.04.2010
*
* @brief   Interface class for the BPNGClient DLL
*/
//******************************************************************************
//******************************************************************************

#ifndef IBPNGCLIENT_H
#define IBPNGCLIENT_H

#include "BPNGDefines.h"
#include "RdbDefines.h"
#include "IClientProperties.h"
#include "IBPNGClientListener.h"

extern "C" {
    //==================================================================================================
    /**
    \brief Interface class for the Telemotive Client Library

    IBPNGClient is the interface class of the blue PiraT Client library.
    To get access to any Telemotive data logger except "blue PiraT 1"  you need a pointer to an inplementing instance of the IBPNGClient interface.
    Use getBPNGClient() to get such a pointer. This will create an implementing instance on the heap.
    To avoid conflicts between different runtime libraries it is obligatory to release this object with
    its IBPNGClient::release() function when not needed any more. Don't call the delete operator directly
    on this pointer.

    To get access to a device chain combined via Telemotive System Link (TSL) you also need a pointer to
    an implementing instance of IBPNGClient interface. Since version 4.1.1 you can use the same factory function getBPNGClient().
    The prior method getTSLClient(int numTSLMember) is no longer available. Please see the documentation of
    the new methods setDevice(), setTSLCluster() and setOfflineData() regarding this issue.
    */
    //==================================================================================================


    struct IBPNGClient
    {
        //==================================================================================================
        // Connection functions

        /// Scan network for logger
        /**
        This function sends one broadcast UDP messages via all network adapters
        and notifies the calling application about responding devices with the
        listener functions onBPNGDeviceDetected(), onBPNGDeviceDisappeared() and
        onBPNGDeviceStateChange() (see IBPNGClientListener.h). For each broadcast
        message sent, the function waits for 600ms for responding devices

        The first function call notifies about all found devices. All following
        calls on the same IBPNGClient instance will only notify about changes to
        the previous call.
        */
        virtual void WINAPI scanNetworkForLogger() = 0;

        /// Activates the detection of devices connected to a different subnet.
        /**
         Logger connected to a different subnet must be configured as DHCP client and
         must have anabled the PingToClient option to be detectable via the scanNetworkForLogger() function.

         Calling this function will start a backgroud thread that waits for incoming pings of such devices.
        */
        virtual void WINAPI activateGatewayLoggerDetection() = 0;

        /// Sets the device, the IBPNGClient instance should operate with (download data, change configuration, update firmware, etc.)
        /**
         Since Lib version 4.1.1 the proceeding who to connect to a device changed. Instead of passing the device resp. its IP address you
         want to operate with to the connect() function, the device must now be set with this function before calling connect().
         If you don't use an OnlineLoggerInfo received via IBPNGClientListener::onBPNGDeviceDetected, only the IP parameter of OnlineLoggerInfo
         is obligatory.
         Example:
             \code {.cpp}
             IBPNGClient* client = getBPNGClient();
             OnlineLoggerInfo device = createEmptyOnlineLoggerInfo();
             device.ip = "192.168.0.233";
             client->setDevice(device);
             client->connect();
             \endcode
            @sa setTSLCluster, setOfflineData
         */
        virtual void WINAPI setDevice(const OnlineLoggerInfo &device) = 0;

        /// Sets the TSL cluster, the IBPNGClient instance should operate with (download data, change configuration, update firmware, etc.)
        /**
          Since Lib version 4.1.1 the proceeding who to connect to a TSL changed. Instead of passing the devices directly to the connect() function,
          the TSL must now be set with this function before calling connect().
          You can use objects of TSLClusterImpl provided with the library package or create your own TSLCluster instance.
          Example:
          \code {.cpp}
              IBPNGClient* client = getBPNGClient();
              OnlineLoggerInfo dev1 = createEmptyOnlineLoggerInfo();
              dev1.ip = "10.23.224.178";
              OnlineLoggerInfo dev2 = createEmptyOnlineLoggerInfo();
              dev2.ip = "10.23.224.56";
              TSLClusterImpl cluster;
              cluster.addDevice(dev1);
              cluster.addDevice(dev2);
              client->setTSLCluster(cluster.getTSLCluster());
              client->connect();
          \endcode
        */
        virtual void WINAPI setTSLCluster(TSLCluster cluster) = 0;

        /// Sets the path to offline data set the IBPNGClient instance should operate with (conversion)
        /**
            Since Lib version 4.1.1 the proceeding who to process offline data changed. Instead of passing the file path directly to the initOnline() function,
            the file path must now be set with this function before calling initialize().

        */
        virtual BOOL WINAPI setOfflineData(const char* path) = 0;

        /// Connect to device or TSL set by \a setDevice() or setTSLCluster()
        virtual BOOL WINAPI connect() = 0;


        /// Connect to passed loggers
        /**
        \ingroup DownloadFunctions
        \ingroup ConversionFunctions

        While the loggers are connected, they won't go to standby mode until the last
        IBPNGClient instance is disconnected.
        If connect fails the function will return 0. On success the return value is 1.
        In case of failure further information can be retrieved with getLastError().

        @param numLogger the number of passed OnlineLoggerInfo devices
        @param devices pointer to first OnlineLoggerInfo
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI connectLogger(int numLogger, OnlineLoggerInfo* devices) = 0;

        /// Disconnect the currently connected logger
        virtual void WINAPI disconnectLogger() = 0;

        /// Check the logger connection, returns 1 for valid connection and 0 for no or broken connection
        virtual BOOL WINAPI isConnected() = 0;


        //==================================================================================================
        // Initialization

        /// Static registration of all messages and formats.
        virtual void WINAPI registerAll() = 0;

        /// Initialization of download and conversion process.
        /**
        For trace download and conversion this function must be called first.
        When operating on a network device, note you have to call the connect() function before.

        Within this function the reference data base is read.
        Please note that reading a large RDB may take some time, especially in debug mode.

        Function will return 0 on failure and 1 on success.
        In case of failure further information can be retrieved with getLastError().

        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI initialize() = 0;

        //==================================================================================================
        // Download functions

        /// Download trace data
        /**
        Pass the size and the pointer to an array of DataSpan. Each span specifies either a time span or an
        index span from the reference data base's entry IDs (DataBaseEntryId).
        IBPNGClient::initialize() must have been called before.

        Function will return 0 on failure, 1 on success and -1 on user abort.
        In case of failure further information can be retrieved with getLastError().

        If getLastError() returns BPNG_DOWNLOAD_ERRORS several errors occured.
        Use getNumDownloadErrors() and getDownloadError(int index) for detailed information.

        @param numSpans Size of the passed DataSpan array in second parameter
        @param dataSpans Array of DataSpan, specifying the time or ID spans that should be downloaded
        @param target Path to the target directory or ZIP file. A passed directory must be empty or not existing. A passed ZIP path must not exist.
        @param doSorting Specifies whether the traces from different logger-internal sources should be sorted to one output stream or not.
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI downloadDataSpans(uint16_t numSpans, DataSpan* dataSpans, const char* target, BOOL doSorting ) = 0;

        // Download trace data
        /**
        Downloads all available trace data younger than \a startTimeStamp from connected device and continues downloading all new 
        captured traces periodically until an added listener returns zero in the implementation of IBNGClientListener::onDownloadProgress().
        
        @param startTimeStamp Traces younger than this time stamp will be downloaded
        @param target Target path where offline data should be stored. Can be a directory or ZIP archive. If directory or ZIP name contains
                      placeholder 'ENDTIME', the file/folder name will be renamed when download was stopped by replacing 'ENDTIME' with the maximum downloaded trace time stamp
        */
        virtual int WINAPI startLiveDownload(uint64_t startTimeStamp, const char* target) = 0;

        //==================================================================================================
        // Conversion functions

        /// Returns the pointer to a new conversion set. \deprecated, use static function createNewConversionSet() instead
        virtual IConversionSet* WINAPI createNewConversionSet() = 0;

        /// Convert all data specified by conversionSet.
        /**
        Before data from a logger or an offline data set can be converted, IBPNGClient::initialize() must have been called before.

        The data specified by conversionSet is converted to the passed target directory.

        Function will return 0 on failure, 1 on success and -1 on user abort.
        In case of failure further information can be retrieved with getLastError().

        If getLastError() returns BPNG_CONVERSION_ERRORS several errors occured.
        Use getNumConversionErrors() and getConversionError(int index) for detailed information.

        @param conversionSet conversion settings, see IConversionSet
        @param target target directory for the converted trace files.
        Depending on the passed ClientProperties the files may be stored in sub folders named by date
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI convertData(IConversionSet* conversionSet, const char* target) = 0;


        //==================================================================================================
        //  config functions

        /// Download the current logger configuration to the passed path
        /**
        If you download the current configuration from the data logger you get a zip Archive that
        contains all relevant XML and XSD files to modifiy the configuration in a valid way and
        reconfigurate the device with the reconfigLogger() function.

        <b>
        On TSL instance you have to pass a base path. All participants logger configurations will be saved
        as zip in that directory including a TSLConfig.txt file with additional informations.
        </b>

        Please note: It is up to you to ensure a valid configuration if you want to modify it with
        your own tools. You should only modify the xml and not the xsd files. "DeviceConfiguration.xml"
        and "FirmwareConfiguration.xml" should also not be modified. They specifiy all xml files that
        are mandatory to reconfigurate the data logger.
        You can validate the xml files with the supplied xsd files and a XML library of your
        choice. One possibility would be the XERCES library, see http://xerces.apache.org/xerces-c/

        @param path The path inclusive file name where to store the downloaded configuration ZIP file or on
        TSL the basepath for config zips
        */
        virtual BOOL WINAPI getConfig(const char* path) = 0;

        /// Reconfig logger with the zipped new configuration
        /**
        Reconfigurates the logger/tsl with the passed configurations. The ZIP archive can be either one that
        was downloaded with the getConfig() method, stored by the client software or a modified one.
        If you want to create your own configuration ZIP archive the structure of this file must be the
        same as of those mentioned above (xml files inside an "etc" directory). The abstract.txt file
        and all *.xsd files are optional. The filename must include the current date in followed form:
        [YYYY-MM-DD_HH-MM-SS] -> Y=year, M=month, D=day, H=hour, M=minute, S=second

        With the OnlineLoggerInfoStringPair structure you can assign the several configurations to the devices.
        OnlineLoggerInfoStringPair.key = OnlineLoggerInfo
        OnlineLoggerInfoStringPair.value = path to local config file
        @sa OnlineLoggerInfoStringPair

        Please note: It is up to you to ensure a valid configuration if you want to modify it with
        your own tools. You should only modify the xml and not the xsd files. "DeviceConfiguration.xml"
        and "FirmwareConfiguration.xml" should also not be modified. They specifiy all xml files that
        are mandatory to reconfigurate the data logger.
        You can validate the xml files with the supplied xsd files and a XML library of your
        choice. One possibility would be the XERCES library, see http://xerces.apache.org/xerces-c/

        @param numLogger Number of following OnlineLoggerInfoStringPair (should be equal to the number of devices on TSL)
        @param loggerToConfigPathPairs Pointer to first OnlineLoggerInfoStringPair
        @param xsdVersionOfConfig xsd version of config version, required if the version differs from the logger config version. Requires FW 4.1.1 or higher.
        The xsd version can be found in the FirmwareConfiguration.xml.
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI reconfigLogger(int numLogger, OnlineLoggerInfoStringPair *loggerConfigPathPairs, const char* xsdVersionOfConfig = nullptr) = 0;

        /// Reconfig logger/TSL with the default configuration
        /**
        An invalid configuration will set the logger/TSL in error state. To fix this one possibility
        is to set the logger's default configuration. On TSL every logger will be reset to default
        configuration.

        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI setDefaultConfig() = 0;


        //==================================================================================================
        //  miscellaneous functions

        /// Get list of all events from the RDB.
        /**
        The events of the device's or offline data set's RDB is returned. IBPNGClient::initialize() must have been called before
        @return Pointer to a IRdbEventList
        */
        virtual IRdbEventList* WINAPI getEventList() = 0;

        /// Get list of all trace blocks from the RDB.
        /**
        @return Pointer to a IRdbEventList
        */
        virtual IRdbTraceBlockList* WINAPI getTraceBlockList() = 0;

        /// Synchronizes the RDB
        /**
        After calling initialize() once you can use this function to synchronize the RDB that getEventList() and getTraceBlockList()
        will return the updated lists.
        */
        virtual BOOL WINAPI synchronizeRdb() = 0;

        /// Return the instance name passed to the getBPNGClient() function
        virtual const char* WINAPI getInstanceName() = 0;

        /// Returns the instance ID that is unique for all IBPNGClient instances in one process
        virtual int WINAPI getInstanceId() = 0;

        /// Get path to the reference data base
        /**
        After calling IBPNGClient::initialize() this function returns the path to the current Reference Data Base
        of the logger resp. the offline data set. For online processes, the RDB is downloaded from the
        logger to a tmp directory. For offline processes from a ZIP archive, the RDB is extracted to a tmp directory.
        For offline processes from a directory this function just returns the path to the RDB inside this directory.

        @return Path to the downloaded or extracted RDB file
        @sa initialize()
        */
        virtual const char* WINAPI getReferenceDataBasePath() = 0;

        /// Get path to the config directory (after calling one of the init functions)
        /**
        After calling IBPNGClient::initialize() this function returns the path to the current extracted
        configuration of the logger resp. the offline data set.
        <b>
        On TSL instance you get paths to the config folders of every participant separated by ';':
        <configpath:1>;<configPath:2>;...
        for example
        C:\\...\\Telemotive_bp2\\PNGINST25452\\BPTSL_10.64.76.171_2\\BP2Img_10.64.76.171_3_PID5452;C:\\...\\Telemotive_bp2\\BPNGINST25452\\BPTSL_10.64.76.171_2\\BP2Img_10.64.76.206_4_PID5452;
        </b>
        @return Path to the folder containing the extracted config archive.
        */
        virtual const char* WINAPI getConfigPath() = 0;

        /// Get name of device
        /**
        After calling IBPNGClient::initialize() this function returns the currently configured device name.
        On TSL the device names will be separated by ';'
        @return The device name
        @sa initialize()
        */
        virtual const char* WINAPI getDeviceName() = 0;

        /// Returns pointer to a channel list interface.
        /**
        After calling IBPNGClient::initialize() this function returns a pointer to the logger's/TSL resp. offline data set's channel list.

        In case of error null is returned and further information can be retrieved with getLastError().

        @sa IChannelList
        */
        virtual const IChannelList* WINAPI getLoggerChannels() = 0;

        /*
        * Deliver BMW information and associated channels.
        * The list contains channels only if their containerId >= 0 and therefor configured.
        * The channelList delivered by IBPNGClient::getLoggerChannels() exists at this moment
        @sa ITesttoolsChannelList
        */
        virtual const ITesttoolsChannelList* WINAPI getLoggerTesttoolsChannels() = 0;

        /// Get the firmware and hardware version.
        /**
        On TSL ensure the versionPairs structure has reserved enough space for all members!

        the versionPairs.value will be the firmware and version string
        the versionPairs.key.mbnr will be the referenced device. Only the mbnr field will be filled, the other fields will be empty!

        @param versionPairs structure description in bpngdefines
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI getVersions(OnlineLoggerInfoStringPair *versionPairs) = 0;

        /// Get the current tmt version string
        virtual const char* WINAPI getTMTVersion() = 0;

        /// Update firmware
        /**
        This function updates the logger's firmware. An internal version check is done.
        If the second parameter \a force is 0 only firmware components with an older
        version then the component's version inside the firmware packet will be updated.

        @param loggerToFirmwareUpdatePacketPair A pair with key=OnlineLoggerInfo, the device to be updated and value=local path to firmware packet
        @param force Flag whether to update the components independently from the components' versions
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI updateFirmware(OnlineLoggerInfoStringPair *loggerFirmwareUpdatePacketPair, BOOL force) = 0;

        virtual BOOL WINAPI updatePBFirmware(OnlineLoggerInfoStringPair *loggerFirmwareUpdatePacketPair, BOOL force) = 0;

        virtual const char* WINAPI getInternalPBVersion(OnlineLoggerInfoStringPair *loggerFirmwareUpdatePacketPair) = 0;

        //function used for user's authentification.
        virtual BOOL WINAPI isUserAuthentificated(PwdPrivilegesFuncId actionName) = 0;

        /// Update licenses
        /**
        Overwrites the current license file with the new one.

        @param loggerLicenseFilePair A pair with key=OnlineLoggerInfo, the device to be updated and value=local path to licsene file
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI updateLicenses(OnlineLoggerInfoStringPair *loggerLicenseFilePair) = 0;

        /// Returns the license file's content of the specified device as string.
        /**
        @param deviceMbnr target device mainboardnumber
        @return license file's content as string
        */
        virtual const char* WINAPI getLicenses(unsigned deviceMbnr) = 0;

        /// Removes the current license file from the logger.
        /**
        Removes the current license file from the logger.

        @return true on success, false on failure
        */
        virtual BOOL WINAPI removeAllLicenses() = 0;


        /// Delete trace data
        /**
        Pass the size and the pointer to an array of RDB-DataBaseEntryIDs all from RdbEvents with RdbEventType 'STARTUP'.
        If you want to retrieve the section startupIds you have to call the initialize() function
        first to get the current RDB file.

        Function will return 0 on failure, 1 on success and -1 on user abort.
        In case of failure further information can be retrieved with getLastError().
        Note that this function is not available for blue PiraT Collect devices (BPNGDeviceType DEV_TRACE_COLL_DS).

        @param numStartUpIds Size of the passed uint64_t array in second parameter
        @param startupIds Array of uint64_t, specifying the startupIds of the sections that should be deleted
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI deleteSectionsByStartUpIds(uint16_t numStartUpIds, uint64_t* startupIds) = 0;

        /// Delete all trace data on the logger
        /**
        In case of failure further information can be retrieved with getLastError().
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI deleteAllData() = 0;

        /// Set an info event with the passed string on the connected logger.
        /**
        You can set an info event to the RDB. This event will be from type INFO and
        the passed message is written to the event's comment column

        @return Returns 0 on failure, 1 on success
        */
        virtual BOOL WINAPI setInfoEvent(const char* msg) = 0;

        /// Set a marker on the connected logger. Returns 0 on error.
        /**
        You can set an marker to the RDB. The set event will be from type MARKER.
        On TSL the marker will be broadcasted internally.

        @return Returns 0 on failure, 1 on success
        */
        virtual BOOL WINAPI setMarker() = 0;

        /// Returns the current loggertime in seconds since 01.01.1970 UTC
        virtual int WINAPI getCurrentLoggerTime() = 0;

        /// Set logger time and date to the passed UTC time stamp
        /**
        The parameter time must be in seconds since 01.01.1970 UTC.
        On TSL the new time will be applied on every device.
        @return -1 on clientLib busy, 0 on failure, 1 on success
        */
        virtual int WINAPI setTime(int time) = 0;

        /// Call this to keep logger alive
        /**
        The blue PiraT 2 data logger can be configured to go to standby after a
        specified timeout without any bus traffic on the connected interfaces.
        If you want to have access to a device without bus traffic, and you don't
        want to connect to it with connectLogger() you have to
        keep it alive by calling this function. This will start a separate thread
        that sends periodically ping messages to the passed IP address. Receiving
        these ping messages, the firmware will not shutdown the system.

        @param ip The IP address of the logger that should be kept alive
        @sa stopKeepLoggerAlive()
        */
        virtual void WINAPI keepLoggerAlive(const char* ip) = 0;

        /// Called to stop sending keep alive pings to the logger specified via the passed \a ip
        virtual void WINAPI stopKeepLoggerAlive(const char* ip) = 0;


        /// Retreive an OnlineLoggerInfo for a particular IP address
        virtual OnlineLoggerInfo* WINAPI getOnlineLoggerInfo(const char* ip) = 0;

        /// Return pointer to a format list interface. Returns null in case of error
        /**
        All formats returned by this function are available for data conversion.
        @sa IFormatList, IFormatInfo
        */
        virtual IFormatList* WINAPI getAvailableFormats() = 0;

        /// Let the connected device blink its front LEDs for identification
        /**
        You can use this function to identify you device if you can't identify it
        over the Name or IP address given from the IBPNGClientListener::onBPNGDeviceDetected callback function.
        On TSL all device LEDs will flash.
        */
        virtual void WINAPI flashDeviceLED() = 0;

        /// Parse CCPXCPMeasurement.xml and CCPXCPConfiguration.xml and create CCPXCPSequence.xml
        virtual int WINAPI createCCPXCPSeqFile(const char* xsdDir, const char* xmlDir, bool forceFlag) = 0;

        /// Parse CCPXCPMeasurement.xml and CCPXCPConfiguration.xml and create a Vector DBC file for each device
        virtual int WINAPI createCCPXCPDbcFiles(const char* dbcDir, const char* xsdDir, const char* xmlDir) = 0;

        /// Return pointer to a false measure signal list interface.
        /**
        After calling the IBPNGClient::createCCPXCPDbcFiles() this function returns a pointer to a list with
        all measure signals which were ignored at DBC file generation.

        @sa IFalseMeasureSignal
        */
        virtual const IFalseMeasureSignalList* WINAPI getFalseMeasureSignals() = 0;

        /// Add a listener
        virtual void WINAPI addListener(IBPNGClientListener* listener) = 0;

        /// Remove a listener
        virtual void WINAPI removeListener(IBPNGClientListener* listener) = 0;

        /// Get last error code
        /**
        If any called BPNGClient function returns a value that indicates an error
        you can retrieve further information about that error with this function.
        @return The error description with error code and optional string value.
        @sa BPNGError
        */
        virtual BPNGError WINAPI getLastError() = 0;

        /// Returns the number of errors occured during the last conversion process.
        /**
        If convertData() fails, getLastError() can return different kinds of errors.
        There are types of errors that won't interrupt the conversion process but will be
        gathered during conversion and notified at the end. In that case the error code
        returned by getLastError() will be BPNG_CONVERSION_ERRORS and you can get the
        number of errors with this function.
        @sa getConversionError()
        */
        virtual int WINAPI getNumConversionErrors() = 0;

        /// Returns the conversion error at \a index.
        /**
        After getting the number of conversion errors with getNumConversionErrors()
        you can get all single errors with this function.
        */
        virtual BPNGError WINAPI getConversionError(int index) = 0;

        /// Returns the number of errors occured during the last download process.
        /**
        If downloadDataSpans() fails, getLastError() can return different kinds of errors.
        There are types of errors that won't interrupt the download process but will be
        gathered during download and notified at the end. In that case the error code
        returned by getLastError() will be BPNG_DOWNLOAD_ERRORS and you can get the
        number of errors with this function.
        @sa getDownloadError()
        */
        virtual int WINAPI getNumDownloadErrors() = 0;

        /// Returns the download error at \a index.
        /**
        After getting the number of download errors with getNumDownloadErrors()
        you can get all single errors with this function.
        */
        virtual BPNGError WINAPI getDownloadError(int index) = 0;

        /// Returns the current fw version
        virtual const char* WINAPI getFWVersion() = 0;

        /// Free memory of this IBPNGClient instance
        /**
        With the call of getBPNGClient() a new instance is created on the heap. The user is responsible to free its
        memory if it isn't needed any more. This function calls the delete operator on itself.

        Important note: Any further function call on the IBPNGClient instance after release() was called
        will cause a memory access violation and will crash the application!
        */
        virtual void WINAPI release() = 0;

        // Returns a copy of the current client properties
        /**
        @sa IClientProperties, setClientProperties()
        */
        virtual IClientProperties* WINAPI getClientProperties() = 0;

        // Set client properties
        /**
        @param Pointer to IClientProperties which can be retrieved from the static function \a createNewClientProperties()
        or from \a IBPNGClient::getClientProperties()
        @sa IClientProperties, getClientProperties(), createNewClientProperties
        */
        virtual void WINAPI setClientProperties(IClientProperties* properties) = 0;

        /// Save properties to ini file
        virtual void WINAPI saveProperties(const char* pathToIniFile) = 0;

        /// Load properties from ini file
        virtual void WINAPI loadProperties(const char* pathToIniFile) = 0;

        /// Remove all DBC file assignments \deprecated, use function clearDatabaseFileAssignments() instead
        /**
        <b>
        will be removed with next vesion!
        </b>
        */
        virtual void WINAPI clearDBCFileAssignments() = 0;

        /// Remove all database file assignments
        virtual void WINAPI clearDatabaseFileAssignments() = 0;

        /// Assign a DBC file to a CAN channel. Multiple files for one CAN channel are allowed, but double used message IDs will ignored. \deprecated, use function assignDatabaseFile() instead
        /**
        <b>
        will be removed with next vesion!
        </b>

        @param channelIndexCAN Zero based CAN channel index
        @param dbcFilePath Absolute path to the dbc file
        */
        virtual void WINAPI assignDBCFile(int channelIndexCAN, const char* dbcFilePath) = 0;

        /// Assign a database file to a channel. Multiple files for one channel are allowed, but double used message IDs will ignored.
        /**
        Supported database formats and channelTypes:
        CH_CAN (also CAN-FD): FIBEX, DBC, Autosar XML
        CH_FLEXRAY: FIBEX, Autosar XML
        CH_MII: Autosar XML
        Supported FIBEX versions: 3.1.0, 3.1.1, 4.0.0, 4.1.0, 4.1.1
        Supported Autosar XML versions: 3.1.4.AI.4, 3.2.1, 3.2.2, 3.2.3, r4.0
        */
        virtual void WINAPI assignDatabaseFile(ChannelType channelType, int channelIndex, const char* databaseFilePath) = 0;

        /// Reset marker counter
        virtual int WINAPI resetMarkerCounter() = 0;

        /**
        set the password file on device specified by the mainboardnumber

        @param path local path of password file
        @param targetMbnr the target device mainboardnumber
        @return 0 on failure, 1 on success
        */
        virtual int WINAPI setPwdFile(const char* path, unsigned targetMbnr) = 0;

        /**
        get the password file of device specified by the mainboardnumber

        @param sourceMbnr the source device mainboardnumber
        @return local path to file
        */
        virtual const char* WINAPI getPwdFile(unsigned sourceMbnr) = 0;

        /**
        check if the device supports password protection

        @param deviceMbnr the device
        @return 1 if the device supports password protection
        */
        virtual BOOL WINAPI isPasswordProtectionSupported(unsigned deviceMbnr) = 0;

        /// Download bug report
        /**
        The downloaded bug report is a ZIP archive with several log data and system files for
        error analyzing purposes.

        @param targetPath Path inclusive file name under that the bug report will be stored.
        @param mode that specifies what kind of data should be included in the report, @sa BPNGBugreportMode
        @param startTime Start time for the time span of trace data that should be included (usec since 01.01.1970 UTC). Only for mode BR_FULL_ALL_TRACES and BR_FULL_TIMESPAN_TRACES
        @param endTime End time for the time span of trace data that should be included (usec since 01.01.1970 UTC). Only for mode BR_FULL_ALL_TRACES and BR_FULL_TIMESPAN_TRACES
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI downloadBugReport(const char* targetPath, BPNGBugreportMode mode, uint64_t startTime, uint64_t endTime) = 0;

        /// restarts the device or TSL
        /**
        @param waitForRestart if 1 communication waits for the restart
        @return 0 on failure, 1 on success, -1 on false fw version
        */
        virtual int WINAPI restartDevice(BOOL waitForRestart) = 0;

        /// shut down the device or TSL
        /**
        @return 0 on failure, 1 on success, -1 on false fw version
        */
        virtual int WINAPI shutdownDevice() = 0;

        /// get memory fill level of device
        /**
        On TSL ensure the fillLevel structure has reserved enough space for all members

        @param fillLevel structure description in bpngdefines
        @return 0 on failure, 1 on success
        */
        virtual BOOL WINAPI getMemoryFillLevel(MemoryFillLevel* fillLevel) = 0;

        /// Converts all time stamps in an offline data set's file names to local time
        virtual BOOL WINAPI convertFileNameTimeStampsToLocalTime(const char* pathToOfflineDataSet) = 0;

        /// Signal filtering
        /**
        This function parses all data of the offline data set that was previously set via setOfflineData() and filters
        signals according to the complex filter settings created with the System Client.
        @param pathToFilterSetting path to the ZIP file including the complex filter settings created with System Client
        @param targetPath path to the target directory where the filtered data should be written to
        */
        virtual BOOL WINAPI filterSignals(const char* pathToFilterSettings, const char* targetPath) = 0;

        /// Signal filtering
        /**
        This function parses all data of an offline data set and filters signals according to the complex filter settings
        created with the System Client.
        @param pathToOfflineDataSet path to the offline data set
        @param pathToFilterSetting path to the ZIP file including the complex filter settings created with System Client
        @param targetPath path to the target directory where the filtered data should be written to
        */
        virtual BOOL WINAPI filterSignalsFromOfflineData(const char* pathToOfflineDataSet, const char* pathToFilterSettings, const char* targetPath) = 0;


        /// easy track test report creation
        /**
        This function creates test reports for every section started by START_TESTDRIVE and ended by END_TESTDRIVE. Every test report will get a own folder in the target directory,
        every timespan on conversionset is a failure and gets it own folder in the test report folder. All selected data in the conversion set will be converted in that failure
        folder.
        @param conversionSet conversion settings, see IConversionSet
        @param target target directory for the test reports.
        @return 0 on failure, 1 on success and -1 on user abort.
        */
        virtual int WINAPI createTestReport(IConversionSet* conversionSet, const char* target) = 0;

        /// enable ClientLogUtil output
        /**
        This function enable the output of class ClientLogUtil.
        */
        virtual void WINAPI enableClientLogOutput() = 0;
    };

    /// Returns the current client library version
    DECLDIR const char* WINAPI getLibVersion();


    /// Factory function that creates instances of BPNGClient giving away ownership.
    /**
    The instance is created on the heap and the allocated memory must be freed by the calling application.
    You can pass a name to this function. This will be the name of the created instance.
    @sa IBPNGClient::release(), IBPNGClient::getInstanceName()
    */
    DECLDIR IBPNGClient* WINAPI getBPNGClient(const char* name = "");

    /**
    Read out the number of TSL members from a offline data set.
    */
    DECLDIR BPNGErrCode WINAPI getNumTSLMemberFromOfflineDataSet(const char* offlinePath, int* numMember);


    /// Sets the directory where all temporary files are created. If not called, the default system's tmp dir is used
    DECLDIR void WINAPI setTempDir(const char* tmp);
    DECLDIR const char* WINAPI getTempDir();

    /// Sets the language for status messages
    DECLDIR void WINAPI setLanguageID(LanguageID id);

    /// returns a new created conversionset
    DECLDIR IConversionSet* WINAPI createNewConversionSet();

    /// To free memory of a IConversionSet, use this method. Otherwise the memory will be freed when detaching the DLL from process.
    /// Never call any function of an IConversionSet after passing the pointer to the freeConversionSetMemory function.
    /// This would cause a heap corruptions.
    DECLDIR void WINAPI freeConversionSetMemory(IConversionSet* convSet);

    // Creates and returns a pointer to a new ClientProperties object with default settings
    /**
    After modifying the properties, you can set them to an instance of
    IBPNGClient with setClientProperties();
    @sa IClientProperties, setClientProperties()
    */
    DECLDIR IClientProperties* WINAPI createNewClientProperties();

    /// To free memory of IClientProperties, use this method. Otherwise the memory will be freed when detaching the DLL from process.
    /// Never call any function of an IClientProperties pointer after passing the pointer to the freeClientPropertiesMemory function.
    /// This would cause a heap corruptions.
    DECLDIR void WINAPI freeClientPropertiesMemory(IClientProperties* prop);

    /// From version 2.1.1 on the client library doesn't write log messages to std::cout by default.
    /// The lib actually doesn't write a log at all unless this function is called.
    /// A log file is created under the passed \a path. If the file already exists, the logs
    /// will be appended. The file will be closed when the DLL is detached from the process.
    DECLDIR void WINAPI writeLogFile(const char* path, int maxSizeInByte, int numBackupFiles);

    /// The library's log output can also be written to std::cout.
    /// If this is required activate cout log with this function. Default is no cout ouput.
    DECLDIR void WINAPI writeLogToCout(bool flag);

    // For debug purpose all logs can also be forwarded to the microsoft OutputDebugString() functions.
    // Those outputs can be catched from the SYSINTERNALS SOFTWARE DebugView.exe (www.sysinternals.com)
    DECLDIR void WINAPI writeLogToDebugView(bool flag);

    /// Adds a log listener to the library
    /**
    If you want to receive the debug outputs from the client library,
    you can set a log listener to the lib. All set listeners get the log outputs
    from all BPNGClient instances.

    All log outputs are forwarded to the registered listeners by calling
    the onLogRequest function that was added.

    @sa onLogRequest
    */
    DECLDIR void WINAPI addLogListener(onLogRequest logFunc);

    /// Removes a log listener from the library
    DECLDIR void WINAPI removeLogListener(onLogRequest logFunc);

    /**
    Creates an empty OnlineLoggerInfo.
    Use this function for devices if the LoggerDetector won't work in your network.

    Don't forget to fill the neccessary fields.

    @sa OnlineLoggerInfo
    */
    DECLDIR OnlineLoggerInfo createEmptyOnlineLoggerInfo();

} // end of extern_C

#endif  // IBPNGCLIENT_H
