//************************************************************
//
// BPNGLoggerDetector.hh
//
/** @file BPNGLoggerDetector.hh
 *
 * @brief IBPNGClientListener wrapper
 */
//************************************************************

#ifndef BPNGLOGGERDETECTOR_HH
#define BPNGLOGGERDETECTOR_HH

// sys
#include <vector>

// client
#include <IBPNGClientListener.h>
#include <IBPNGClient.h>
#include <BPNGDefines.h>
#include <OnlineLoggerInfoWrapper.hh>

// Test
#include <TSLClusterImpl.hh>


//************************************************************
//
// BPNGLoggerDetector
//
//************************************************************

class BPNGLoggerDetector : public IBPNGClientListener
{
public:

    /// CTOR
    template <typename IP>
    BPNGLoggerDetector(IP &&_ip, unsigned int _debug);

    ~BPNGLoggerDetector()
    {
    }

    // Listener functions from IBPNGClientListener
    virtual void onBPNGDeviceDetected(OnlineLoggerInfo *info);
    virtual void onBPNGDeviceDisappeared(OnlineLoggerInfo *info);
    virtual void onBPNGDeviceStateChange(OnlineLoggerInfo *info);
    virtual int onProgressDataDownload(int percentCompleted);
    virtual int onProgressDataDownload(int percentCompleted, uint64_t downloadedSize, uint64_t totalSize);
    virtual int onProgressConversion(int percentCompleted, const char *status);
    virtual int onProgressDeletion(int percentCompleted);
    virtual int onTargetPathTooLong(char *newTarget, int maxSize);
    virtual int getOverwritingPermission(const char *filePath);
    virtual void onFirmwareUpdateProgress(int percentage, int step, int substep, const char *desc);
    virtual void onFirmwareUpdateError(int errorId);
    virtual int onCriticalDiskSpace(uint64_t freeSpace, uint64_t neededSpace, const char *drive, const char *msg);
    virtual void onLoggerConnected(bool isConnected);
    virtual void onStatusMessage(const char *statusMsg);
    virtual int onGetLogReportProgress(int percentage, const char *desc);
    virtual void onWarning(BPNGWarningCode warningCode, const char *warnMsg);
    virtual void onInvalidPwConfigFound(const char *);
    virtual void onDownloadStart(int64_t totalAmountOfBytes);
    virtual void onLogInDataFailed();
    virtual void onResetLogInDataFailed();
    virtual void onFuncAccessDenied();
    virtual void onConversionStart(int64_t totalAmountOfBytes);
    virtual int onDataRecoverProgress(const char *test1, int test2) { return 0; }
    virtual const char * onLogInDataRequired(unsigned int test1) { return nullptr; }
    virtual void onInvalidPwConfigFound(unsigned int) {}
    virtual const char * onExtractionPasswordRequired(unsigned int) { return nullptr; }
    virtual bool isTerminateLiveDownloadRequest() { return false; }

    OnlineLoggerInfo getLoggerInfoForIP(const char *ip) const;

    const std::string & getIP() const
    {
        return ip;
    }

    /**
       Returns a vector of detected BPNGDevice in local networks.

       @param searchTimeOut the search timeout in seconds
       @return vector of BPNGDevice
    */
    std::vector<OnlineLoggerInfoWrapper> getLoggerList(unsigned searchTimeOut);

    /**
       Checks a vector of BPNGDevice for TSL chains. Combines the devices with same tslNetworkId (except -1)
       to TSLCluster.

       @param loggersInNetwork the BPNGDevice in network found by getLoggerList(unsigned searchTimeOut)
       @return vector of TSLCluster
    */
    std::vector<TSLClusterImpl> getTSLs(const std::vector<OnlineLoggerInfoWrapper> &tslChain);

    /**
       Remove RCT from TSL for spezific functions like delete data.

       @param loggersInNetwork  TSL Chain to check for rct devices
    */
    void excludeRCTFromTSL(TSLClusterImpl &loggersInNetwork);


private:

    std::string ip;
    int oldpercent;

    std::vector<OnlineLoggerInfoWrapper> loggerList;

    /// Log callback.
    static void logFunc(const char *s);
};


// inline and template functions

template <typename IP>
inline
BPNGLoggerDetector::BPNGLoggerDetector(IP &&_ip, unsigned int _debug)
    : ip(std::forward<IP>(_ip)),
      oldpercent(-1),
      loggerList()
{
    if (_debug > 0)
        addLogListener(logFunc);
}

#endif  // BPNGLOGGERDETECTOR_HH
