//************************************************************
//
// RdbTraceBlockList.hh
//
/** @file RdbTraceBlockList.hh
 *
 * @brief IRdbTraceBlock wrapper
 */
//************************************************************

#ifndef RDBTRACEBLOCKLIST_HH
#define RDBTRACEBLOCKLIST_HH

#include <vector>
#include <string>

#include "BPNGDefines.h"
#include "RdbDefines.h"


/// Implementation class for a wrapper of IRdbTraceBlock using STL
/// classes.
/**
 * To achieve a compiler independend interface for the blue PiraT 2
 * client library only pointer to complex objects are returned from
 * some functions. The IRdbTraceBlock class can be wrapped by this
 * class RdbTraceBlock to have access to its members in the usual way.
 * You only have to pass a IRdbTraceBlock pointer to the constructor.
 *
 * \sa IRdbEvent, RdbEventList
 */
struct RdbTraceBlock2
{
    RdbTraceBlock2(const IRdbTraceBlock* rdbBlock)
        : m_DataBaseEntryId(rdbBlock->getUniqueId()),
          m_LoggerModuleName(rdbBlock->getLoggerModuleName()),
          m_FilePath(rdbBlock->getFilePath()),
          m_FileName(rdbBlock->getFileName()),
          m_DataFileSize(rdbBlock->getDataFileSize()),
          m_DataSize(rdbBlock->getDataSize()),
          m_DataStartTimeUTC(rdbBlock->getStartTimeStamp()),
          m_DataEndTimeUTC(rdbBlock->getEndTimeStamp()),
          m_BlockNr(rdbBlock->getBlockNumber()),
          m_TimeZone(rdbBlock->getTimeZone()),
          m_CfgBackupFile(rdbBlock->getCfgBackupFile()),
          m_CAN_CANextData(rdbBlock->getDataColumnValue("CAN_CANextData")),
          m_MOST25Data(rdbBlock->getDataColumnValue("MOST25Data")),
          m_SerialData(rdbBlock->getDataColumnValue("SerialData")),
          m_EthernetData(rdbBlock->getDataColumnValue("EthernetData")),
          m_FlexRayData(rdbBlock->getDataColumnValue("FlexRayData")),
          m_LINData(rdbBlock->getDataColumnValue("LINData")),
          m_ApixData(rdbBlock->getDataColumnValue("ApixData")),
          m_MOST150Data(rdbBlock->getDataColumnValue("MOST150Data")),
          m_CameraData(rdbBlock->getDataColumnValue("CameraData")),
          m_AnalogData(rdbBlock->getDataColumnValue("AnalogData")),
          m_GpioData(rdbBlock->getDataColumnValue("GpioData")),
          m_AudioData(rdbBlock->getDataColumnValue("AudioData")),
          m_CCPXCPData(rdbBlock->getDataColumnValue("CCPXCPData")),
          m_DiagData(rdbBlock->getDataColumnValue("DiagData")),
          m_GPSData(rdbBlock->getDataColumnValue("GPSData")),
          m_ECLData(rdbBlock->getDataColumnValue("ECLData")),
          m_CLASSData(rdbBlock->getDataColumnValue("CLASSData")),
          m_ComplexFilterData(rdbBlock->getDataColumnValue("ComplexFilterData")),
          m_TTYData(rdbBlock->getDataColumnValue("TTYData")),
          m_Comment(rdbBlock->getComment())
    {
    }

    uint64_t m_DataBaseEntryId;
    std::string m_LoggerModuleName;
    std::string m_FilePath;
    std::string m_FileName;
    uint64_t m_DataFileSize;
    uint64_t m_DataSize;
    uint64_t m_DataStartTimeUTC;	///< start time of trace block in us
    uint64_t m_DataEndTimeUTC;		///< end time of trace block in us
    std::string m_DataStartGPS;
    std::string m_DataEndGPS;
    uint64_t m_BlockNr;
    std::string m_TimeZone;
    std::string m_CfgBackupFile;
    std::string m_CAN_CANextData;
    std::string m_MOST25Data;
    std::string m_SerialData;
    std::string m_EthernetData;
    std::string m_FlexRayData;
    std::string m_LINData;
    std::string m_ApixData;
    std::string m_MOST150Data;
    std::string m_CameraData;
    std::string m_AnalogData;
    std::string m_GpioData;
    std::string m_AudioData;
    std::string m_CCPXCPData;
    std::string m_DiagData;
    std::string m_GPSData;
    std::string m_ECLData;
    std::string m_CLASSData;
    std::string m_ComplexFilterData;
    std::string m_TTYData;
    std::string m_MIIData;
    std::string m_Comment;
};

/// Implementation class for a wrapper of IRdbTraceBlockList using STL
/// classes
/**
 * To achieve a compiler independend interface for the blue PiraT 2
 * client library only pointer to complex objects are returned from
 * some functions. The class IRdbTraceBlockList is nothing else than a
 * vector of IRdbTraceBlock objects. Pass a pointer to
 * IRdbTraceBlockList to the constructor of this wrapper class
 * RdbTraceBlockList and you get a STL vector of RdbTraceBlock objects
 * which by itself is a wrapper to IRdbTraceBlock
 *
 * \sa RdbTraceBlock, IRdbTraceBlockList, IRdbTraceBlock
 */
class RdbTraceBlockList : public std::vector<RdbTraceBlock2>
{
public:

    RdbTraceBlockList(const IRdbTraceBlockList *list)
    {
        const size_t size = list->getSize();
        reserve(size);
        for (size_t i = 0; i < size; ++i)
        {
            push_back(RdbTraceBlock2(list->getTraceBlock(i)));
        }
    }
        /// Sort trace blocks by their data base entry id
    void sortTraceBlocksById()
    {
        stable_sort(this->begin(), this->end(), TraceBlockIdComp());
    }

private:

    /// Sort algorithm
    struct TraceBlockIdComp : public std::binary_function<RdbTraceBlock2, RdbTraceBlock2, bool>
    {
	bool operator()(const RdbTraceBlock2 &x, const RdbTraceBlock2 &y) const
	{
	    return x.m_DataBaseEntryId < y.m_DataBaseEntryId;
	}
    };
};

#endif  // RDBTRACEBLOCKLIST_HH
