//************************************************************
//
// TSLClusterImpl.hh
//
/** @file TSLClusterImpl.hh
 *
 * @brief TSLCluster wrapper
 */
//************************************************************

#ifndef TSLCLUSTERIMPL_HH
#define TSLCLUSTERIMPL_HH

#include <vector>
#include <cstdlib>
#include <iostream>

#include "BPNGDefines.h"

/**
 * A simple class that represents a Telemotive System Link chain.
 */
class TSLClusterImpl
{
public:

    /// A enumaration of types of which task the connection will be used for.
    enum ConnectionType
    {
        DOWNLOAD,   ///< Download tasks
        CONVERSION, ///< Conversion tasks
        CONFIG,     ///< Configuration tasks
        BUGREPORT,  ///< Create bug report
        FW_UPDATE   ///< make Firmware update
    };


    /// Constructor
    TSLClusterImpl()
    {
    }

    /**
     * Constructor
     * @param firstDevice the first device of the chain.
     */
    TSLClusterImpl(OnlineLoggerInfo firstDevice)
    {
        addDevice(firstDevice);
    }

    /**
     * Add a BPNGDevice to the TSLCluster.
     * @param device the new device
     */
    void addDevice(OnlineLoggerInfo device)
    {
        if (m_tslChain.empty())
        {
            m_tslNetworkId = device.tslNetworkId;
        }

        m_tslChain.push_back(device);
    }

    /**
     * Delete a BPNGDevice from the TSLCluster.
     * @param index  index of device to delete
     */
    void deleteDevice(size_t index)
    {
        if (m_tslChain.size() > index)
        {
            m_tslChain.erase(m_tslChain.begin() + index);
        }
    }

    /**
     * Get the name of TSL chain. All devices in the chain have the same TSL name.
     * @return the TSL name.
     */
    std::string getTSLName()
    {
	std::string name(m_tslChain[0].tslName);
        return name.empty() ? m_tslChain[0].name : name;
    }


    /// Stream the TSLCluster to cout.
    void print()
    {
        for (size_t i = 0; i < m_tslChain.size(); ++i)
        {
            OnlineLoggerInfo device = m_tslChain[i];
            std::cout << "--Dev: [" << device.name << "] IP[" << device.ip << "]\n";
        }
    }

    /**
     * Begin iterator for ranged base for loop
     * @return the begin iterator of internal BPNGDevice vector.
     */
    std::vector<OnlineLoggerInfo>::iterator begin() { return m_tslChain.begin(); }

    /**
     * End iterator for ranged base for loop
     * @return the end iterator of internal BPNGDevice vector.
     */
    std::vector<OnlineLoggerInfo>::iterator end() { return m_tslChain.end(); }

    TSLCluster getTSLCluster()
    {
        TSLCluster cluster;
        cluster.numDevices = m_tslChain.size();
        cluster.loggerArray = m_tslChain.empty() ? nullptr : &m_tslChain[0];
        return cluster;
    }

    /// get size of TSL chain
    size_t getTSLSize() { return m_tslChain.size(); }

private:

    /// The internal vector of BPNGDevice in the TSL.
    std::vector<OnlineLoggerInfo> m_tslChain;

    /// The network id of the TSL.
    int m_tslNetworkId;
};

#endif // TSLCLUSTERIMPL_HH
