//************************************************************
//
/** @file clientUtil.hh
 *
 * @brief Helper functions
 */
//************************************************************


#ifndef UTILITY_HH
#define UTILITY_HH

#include <chrono>
#include <iosfwd>
#include <iostream>
#include <sstream>
#include <time.h>
#include <string.h>
#include <vector>
#include <sys/stat.h>

/// Get the program name, similar to basename.
extern const char *progname(const char *path)
{
    const char *prog;

    if ((prog = strrchr(path, '/')))
	return ++prog;
    else
	return path;
}


template <class Duration>
constexpr std::chrono::hours
toHours(const Duration &d)
{
    return std::chrono::duration_cast<std::chrono::hours>(d);
}


template <class Duration>
constexpr std::chrono::minutes
toMinutes(const Duration &d)
{
    return std::chrono::duration_cast<std::chrono::minutes>(d);
}


template <class Duration>
constexpr std::chrono::seconds
toSeconds(const Duration &d)
{
    return std::chrono::duration_cast<std::chrono::seconds>(d);
}


template <class Duration>
constexpr std::chrono::milliseconds
toMilliseconds(const Duration &d)
{
    return std::chrono::duration_cast<std::chrono::milliseconds>(d);
}


template <class Duration>
constexpr std::chrono::microseconds
toMicroseconds(const Duration &d)
{
    return std::chrono::duration_cast<std::chrono::microseconds>(d);
}


template <class Duration>
constexpr std::chrono::nanoseconds
toNanoseconds(const Duration &d)
{
    return std::chrono::duration_cast<std::chrono::nanoseconds>(d);
}


/// Convert a chrono duration into a POSIX timespec.
template <typename Rep, typename Period>
inline timespec
toTimespec(const std::chrono::duration<Rep, Period> &d)
{
    return timespec {
	static_cast<decltype(timespec::tv_sec)>(::toSeconds(d).count()),
	static_cast<decltype(timespec::tv_nsec)>(::toNanoseconds(d - ::toSeconds(d)).count())};
}

static void printTm(std::ostream &os, time_t sec, char origfill)
{
    tm ptm;

    gmtime_r(&sec, &ptm);

    const std::ios::fmtflags oldflags = os.flags(std::ios::fixed | std::ios::dec);
    const char oldfill = os.fill('0');

    if (origfill == '\0')
    {
	os << std::setw(2) << ptm.tm_hour
	   << std::setw(2) << ptm.tm_min
	   << std::setw(2) << ptm.tm_sec;
    }
    else
    {
	char fill = ':';
	if (origfill == '-' || origfill == '.')
	    fill = origfill;

	os << std::setw(2) << ptm.tm_hour
	   << fill
	   << std::setw(2) << ptm.tm_min
	   << fill
	   << std::setw(2) << ptm.tm_sec;
    }

    os.flags(oldflags);
    os.fill(oldfill);
}

extern std::ostream &operator<<(std::ostream &os, const tm &_t)
{
    const std::ios::fmtflags oldflags = os.flags(std::ios::fixed | std::ios::dec);
    const char oldfill = os.fill('0');

    // ISO format
    os << std::setw(4) << _t.tm_year + 1900 << '-'
       << std::setw(2) << _t.tm_mon + 1 << '-'
       << std::setw(2) << _t.tm_mday << ' '
       << std::setw(2) << _t.tm_hour << ':'
       << std::setw(2) << _t.tm_min << ':'
       << std::setw(2) << _t.tm_sec;

    os.flags(oldflags);
    os.fill(oldfill);

    return os;
}

/// Output operator for struct timespec as used e.g. by pselect or nanosleep.
/** Prints out the time specified in @a _t in seconds. */
extern std::ostream &operator<<(std::ostream &os, const timespec &_t)
{
    const std::ios::fmtflags oldflags = os.flags(std::ios::fixed | std::ios::right);
    const char oldfill = os.fill(' ');

    if (os.flags() & std::ios::scientific)
	os << _t.tv_sec;
    else
	printTm(os, _t.tv_sec, oldfill);

    os << '.' << std::setfill('0') << std::setw(9) << _t.tv_nsec;

    os.flags(oldflags);
    os.fill(oldfill);

    return os;
}

/// Output operator for a chrono duration.
/** If the duration has to be printed in a hh:mm:ss.ms notation you
 * have to use e.g.
 *
 * cout << toTimespec(d)
 */
template <typename Rep, typename Period>
inline std::ostream &
operator<<(std::ostream &os, const std::chrono::duration<Rep, Period> &d)
{
    os << d.count();
    return os;
}

/// Output operator for a chrono duration with period one.
/** Formatting:
 *
 * - std::ios::showbase: the unit "s" will be printed
 *
 * - std::ios::skipws: no space between the time and the unit
 */
template <typename Rep>
std::ostream &
operator<<(std::ostream &os, const std::chrono::duration<Rep, std::ratio<1>> &d)
{
    os << d.count();
    if (os.flags() & std::ios::showbase)
	os << (os.flags() & std::ios::skipws ? "s" : " s");
    return os;
}

/// Output operator for a chrono duration  with Period milli.
/** Formatting:
 *
 * - std::ios::showbase: the unit "ms" will be printed
 *
 * - std::ios::skipws: no space between the time and the unit
 */
template <typename Rep>
std::ostream &
operator<<(std::ostream &os, const std::chrono::duration<Rep, std::milli> &d)
{
    os << d.count();
    if (os.flags() & std::ios::showbase)
	os << (os.flags() & std::ios::skipws ? "ms" : " ms");
    return os;
}

/// Output operator for a chrono duration  with Period micro.
/** Formatting:
 *
 * - std::ios::showbase: the unit "µs" will be printed
 *
 * - std::ios::skipws: no space between the time and the unit
 */
template <typename Rep>
std::ostream &
operator<<(std::ostream &os, const std::chrono::duration<Rep, std::micro> &d)
{
    os << d.count();
    if (os.flags() & std::ios::showbase)
	os << (os.flags() & std::ios::skipws ? "µs" : " µs");
    return os;
}

/// Output operator for a chrono duration  with Period nano.
/** Formatting:
 *
 * - std::ios::showbase: the unit "ns" will be printed
 *
 * - std::ios::skipws: no space between the time and the unit
 */
template <typename Rep>
std::ostream &
operator<<(std::ostream &os, const std::chrono::duration<Rep, std::nano> &d)
{
    os << d.count();
    if (os.flags() & std::ios::showbase)
	os << (os.flags() & std::ios::skipws ? "ns" : " ns");
    return os;
}

/// Output operator for a chrono time point. Prints the date and time
/// since the epoch.
/** The following adjustments can be done:
 *
 * - If std::ios_base::scientific is set in the flags the duration will
 * be printed seconds plus fractions of a second, otherwise it will be
 * printed in a readable format.
 *
 * - If the precision is set to a value of 9 or larger nanosecond
 * precision is printed.
 */
template <typename Clock, typename Duration>
inline std::ostream &
operator<<(std::ostream &os, const std::chrono::time_point<Clock, Duration> &tp)
{
    const auto d = tp.time_since_epoch();

    if (os.flags() & std::ios::scientific)
	os << d;
    else
	os << toTimespec(d);

    return os;
}

extern int mkdirPath(const std::string &_dirpath, int _mode = S_IRWXU | S_IRWXG | S_IRWXO)
{
    if (_dirpath.empty())
    {
	errno = EINVAL;
	return -1;
    }

    struct stat sbuf;

    char *ps;
    std::vector<char> p;

    // create a writeable copy of _dirpath, terminate it with null char
    p.resize(_dirpath.size() + 1);
    std::vector<char>::iterator pend = std::copy(_dirpath.begin(), _dirpath.end(), p.begin());
    *pend = '\0';
    ++pend;

    char * const pstart = &p[0];
    ps = pstart;
    while (ps != &*pend)
    {
	if (ps != pstart)
	{
	    if (*ps == '/' || *ps == '\0')
	    {
		// found path separator, temporarily replace it using a null char
		*ps = '\0';

		// create that part if it doesn't yet exist
		if (stat(pstart, &sbuf) == -1)
		{
		    if (mkdir(pstart, _mode) == -1)
		    {
			return -1;
		    }
		}
		else if (!S_ISDIR(sbuf.st_mode))
		{
		    // it's not a directory, quit
		    errno = ENOTDIR;
		    return -1;
		}

		*ps = '/'; // back to normal ...
	    }
	}

	++ps; // ... and advance
    }

    errno = 0;
    return 0;
}

#endif /* UTILITY_HH */
